/* global vitisApp, goog, angular, bootbox, oVFB */

'use strict';
goog.provide('extraction.script_module');
vitisApp.on('appMainDrtvLoaded', function () {
    /**
     * setWabStatusMethod function.
     * Définition de la méthode de transition.
     * @param {string} sMethodName Nom de la méthode wab.
     **/
    angular.element(vitisApp.appMainDrtv).scope()["setWabStatusMethod"] = function (sMethodName) {
        // Injection des services.
        var $log = angular.element(vitisApp.appMainDrtv).injector().get(["$log"]);
        //var $rootScope = angular.element(vitisApp.appMainDrtv).injector().get(["$rootScope"]);
        var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
        //
        $log.info("setWabStatusMethod", sMethodName);
        envSrvc["oFormValues"][envSrvc["sFormDefinitionName"]]["tr_status_method"] = sMethodName;
        // Envoi le formulaire.
        if (envSrvc["sMode"] != "display") {
            var sFormElementName = envSrvc["oFormDefinition"][envSrvc["sFormDefinitionName"]]["name"];
            var formScope = angular.element("form[name='" + sFormElementName + "']").scope();
            if (envSrvc["sMode"] == "update")
                formScope["sendForm"]();
        }
    };

    /**
     * loadExtractionConfig function.
     * Chargement de la section "Configuration EXTRACTION" dans l''''onglet "Configuration".
     * @expose
     */
    angular.element(vitisApp.appMainDrtv).scope()["loadExtractionConfig"] = function () {
        // Injection des services.
        var $log = angular.element(vitisApp.appMainDrtv).injector().get(["$log"]);
        var envSrvc = angular.element(vitisApp.appWorkspaceListDrtv).injector().get(["envSrvc"]);
        //
        $log.info("loadExtractionConfig");
        // Paramètres des properties dans les valeurs du formulaire de config de Extraction.
        angular.element(vitisApp.appMainDrtv).scope()["setPropertiesFormValues"]();
        // Surcharge l''''url du formulaire des properties de Extraction.
        var scope = this;
        var sTable = envSrvc["oSelectedObject"]["name"];
        scope["oFormRequestParams"] = {
            "sUrl": "modules/extraction/forms/" + envSrvc["oSelectedMode"]["mode_id"] + "/" + sTable + "_" + envSrvc["oSectionForm"][sTable]["sections"][envSrvc["oSectionForm"][sTable]["iSelectedSectionIndex"]]["name"] + ".json"
        };
    };
    
    /**
     * initExtractionExtractionForm function.
     * Traitements avant l'affichage du formulaire de l'onglet "Extraction".
     **/
    angular.element(vitisApp.appMainDrtv).scope()["initExtractionExtractionForm"] = function () {
        // Injection des services.
        var $log = angular.element(vitisApp.appMainDrtv).injector().get(["$log"]);
        var $rootScope = angular.element(vitisApp.appMainDrtv).injector().get(["$rootScope"]);
        var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
        var formSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["formSrvc"]);
        //
        $log.info("initExtractionExtractionForm");
        var oFormValues = envSrvc["oFormValues"][envSrvc["sFormDefinitionName"]];
        var oElemDef;
        // Affiche le champ de form. correspondant au type de méthode choisie.
        var aExtentMethod = ["id_com", "id_conseil", "extent"];
        for (var i in aExtentMethod) {
            var oElemDef = formSrvc["getFormElementDefinition"](aExtentMethod[i], envSrvc["sFormDefinitionName"], envSrvc["oFormDefinition"]);
            if (oFormValues["extent_method"] == aExtentMethod[i])
                oElemDef["visible"] = true;
            else
                oElemDef["visible"] = false;
        }
        // Libellé des champs "Commune" et "Bassin de vie".
        if (envSrvc["sMode"] == "display") {
            oFormValues["id_com"] = oFormValues["id_com_label"];
            oFormValues["id_conseil"] = oFormValues["id_conseil_label"];
        }
        // Attends la fin du chargement de tous les champs du formulaire.
        var clearListener = $rootScope.$on('formExtracted', function (event, sFormDefinitionName) {
            // Supprime le "listener".
            clearListener();
            // Evènement sur le <select> "méthode choisie" : affiche le champ de form. correspondant au choix.
            var oExtentMethod = formSrvc["getFormElementDefinition"]("extent_method", envSrvc["sFormDefinitionName"], envSrvc["oFormDefinition"]);
            if (oExtentMethod["type"] == "select" && document.getElementById(oExtentMethod["id"]) != null) {
                document.getElementById(oExtentMethod["id"]).addEventListener("change", function () {
                    for (var i in aExtentMethod) {
                        var oElemDef = formSrvc["getFormElementDefinition"](aExtentMethod[i], envSrvc["sFormDefinitionName"], envSrvc["oFormDefinition"]);
                        if (this.value == aExtentMethod[i])
                            oElemDef["visible"] = true;
                        else
                            oElemDef["visible"] = false;
                    }
                    var formScope = angular.element("form[name='" + envSrvc["oFormDefinition"][envSrvc["sFormDefinitionName"]]["name"]).scope();
                    formScope.$apply();
                });
            }
        });
    };
    
    /**
     * getExtractionLayersTree function.
     * Retourne l'arborescence pour le champ treeview "layers".
     **/
    angular.element(vitisApp.appMainDrtv).scope()["getExtractionLayersTree"] = function () {
        // Injection des services.
        var $log = angular.element(vitisApp.appMainDrtv).injector().get(["$log"]);
        var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
        var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["propertiesSrvc"]);
        var $q = angular.element(vitisApp.appHtmlFormDrtv).injector().get(["$q"]);
        //
        $log.info("getExtractionLayersTree");
        var deferred = $q.defer();
        var promise = deferred.promise;
        // Charge la liste des couches.
        ajaxRequest({
            "method": "GET",
            "url": propertiesSrvc["web_server_name"] + "/" + propertiesSrvc["services_alias"] + "/extraction/extractions/Layers",
            "scope": angular.element("form[name='" + envSrvc["oFormDefinition"][envSrvc["sFormDefinitionName"]]["name"] + "']").scope(),
            "success": function(response) {
                if (response["data"]["status"] == 1) {
                    // Création de l'arborescence des couches pour le champ "treeview".
                    var aLayers = response["data"]["layers"];
                    var aNodes = [];
                    var stats = true;
                    if (typeof(aLayers) != "undefined") {
                        var aCategories = Object.keys(aLayers);
                        for (var i in aCategories) {
                            var aNode = {"text": aCategories[i], "nodes": []}
                            for (var j in aLayers[aCategories[i]]) {
                                if (aLayers[aCategories[i]][j]["statistique"] !== true)
                                {
                                    stats = false;
                                    aNode["nodes"].push({"text": aLayers[aCategories[i]][j]["text"], "value": aLayers[aCategories[i]][j]["value"], "mutualized": aLayers[aCategories[i]][j]["mutualized"]});
                                }
                            }
                            if (stats !== true)
                            {
                                aNodes.push(aNode);
                            }
                            stats = true;
                        }
                    }
                    
                    this["setTreeviewNodesState"] = function (aNodes) {
                        for (var i in aNodes) {
                            if (typeof (aNodes[i]["nodes"]) != "undefined")
                            {
                                this["setTreeviewNodesState"](aNodes[i]["nodes"]);
                            }
                            else {
                                //Si la couche est mutualisée
                                if (aNodes[i]['mutualized'] == true)
                                {
                                    aNodes[i]["color"] =  propertiesSrvc["color_mutualized"];
                                } else if (aNodes[i]['mutualized'] == false) {
                                    aNodes[i]["color"] =  propertiesSrvc["color_no_mutualized"];
                                }
                            }
                        }
                    }
                    this["setTreeviewNodesState"](aNodes);
                    
                    deferred.resolve(aNodes);
                } else {
                    //
                    var oOptions = {
                        "className": "modal-danger"
                    };
                    // Message d'erreur ?
                    if (response["data"]["errorMessage"] != null)
                        oOptions["message"] = response["data"]["errorMessage"];
                    scope.$root["modalWindow"]("alert", "REQUEST_ERROR", oOptions);
                }
            }
        });
        return promise;
    };
    
    
    /**
     * getExtractionLayersTree function.
     * Retourne l'arborescence pour le champ treeview "layers".
     **/
    angular.element(vitisApp.appMainDrtv).scope()["getExtractionStatsTree"] = function () {
        // Injection des services.
        var $log = angular.element(vitisApp.appMainDrtv).injector().get(["$log"]);
        var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
        var propertiesSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["propertiesSrvc"]);
        var $q = angular.element(vitisApp.appHtmlFormDrtv).injector().get(["$q"]);
        //
        $log.info("getExtractionStatsTree");
        var deferred = $q.defer();
        var promise = deferred.promise;
        // Charge la liste des couches.
        ajaxRequest({
            "method": "GET",
            "url": propertiesSrvc["web_server_name"] + "/" + propertiesSrvc["services_alias"] + "/extraction/extractions/Layers",
            "scope": angular.element("form[name='" + envSrvc["oFormDefinition"][envSrvc["sFormDefinitionName"]]["name"] + "']").scope(),
            "success": function(response) {
                if (response["data"]["status"] == 1) {
                    // Création de l'arborescence des couches pour le champ "treeview".
                    var aLayers = response["data"]["layers"];
                    var aNodes = [];
                    var stats = false;
                    if (typeof(aLayers) != "undefined") {
                        var aCategories = Object.keys(aLayers);
                        for (var i in aCategories) {
                            var aNode = {"text": aCategories[i], "nodes": []}
                            for (var j in aLayers[aCategories[i]]) {
                                if (aLayers[aCategories[i]][j]["statistique"] === true)
                                {
                                    stats = true;
                                    aNode["nodes"].push({"text": aLayers[aCategories[i]][j]["text"], "value": aLayers[aCategories[i]][j]["value"], "mutualized": aLayers[aCategories[i]][j]["mutualized"]});
                                }
                            }
                            if (stats === true)
                            {
                                aNodes.push(aNode);
                            }
                            stats = false;
                        }
                    }
                    
                    this["setTreeviewNodesState"] = function (aNodes) {
                        for (var i in aNodes) {
                            if (typeof (aNodes[i]["nodes"]) != "undefined")
                            {
                                this["setTreeviewNodesState"](aNodes[i]["nodes"]);
                            }
                            else {
                                //Si la couche est mutualisée
                                if (aNodes[i]['mutualized'] == true)
                                {
                                    aNodes[i]["color"] =  propertiesSrvc["color_mutualized"];
                                } else if (aNodes[i]['mutualized'] == false) {
                                    aNodes[i]["color"] =  propertiesSrvc["color_no_mutualized"];
                                }
                            }
                        }
                    }
                    this["setTreeviewNodesState"](aNodes);
                    
                    deferred.resolve(aNodes);
                } else {
                    //
                    var oOptions = {
                        "className": "modal-danger"
                    };
                    // Message d'erreur ?
                    if (response["data"]["errorMessage"] != null)
                        oOptions["message"] = response["data"]["errorMessage"];
                    scope.$root["modalWindow"]("alert", "REQUEST_ERROR", oOptions);
                }
            }
        });
        return promise;
    };
    
    /**
     * appExtractionLayersColumn directive.
     * Mise en forme de la colonne "layers" dans la liste de l'onglet "Extraction".
     * @param {service} $translate Translate service.
     * @ngInject
     **/
    vitisApp.appExtractionLayersColumnDrtv = function ($translate) {
        return {
            link: function (scope, element, attrs) {
                // 1er affichage ou tri de la liste : maj de la mise en forme.
                var clearObserver = attrs.$observe("appExtractionLayersColumn", function (value) {
                    // Si le champ est vide : supprime l'icône.
                    if (scope["row"]["entity"][scope["col"]["field"]] == null || scope["row"]["entity"][scope["col"]["field"]] == "")
                        element[0].className = "";
                    else {
                        // Classes css (ui-grid + spécifiques).
                        element[0].className = "ui-grid-cell-contents info-icon";
                        // Traduction du titre et du contenu.
                        $translate(["EXTRACTION_REF_EXTRACTION_LAYER_LAYERS_TOOLTIP_TITLE"]).then(function (translations) {
                            // Création du "tooltip".
                            $(element)["popover"]({
                                "trigger": "hover",
                                "container": "body",
                                "title": function () {
                                    return translations["EXTRACTION_REF_EXTRACTION_LAYER_LAYERS_TOOLTIP_TITLE"];
                                },
                                "content": function () {
                                    var sValue = scope["row"]["entity"][scope["col"]["field"]];
                                    return sValue.replace(/\|/g, "<br>");
                                },
                                // Placement du tooltip à gauche ou à droite suivant la position horizontale de l'élément.
                                "placement": function (oPopoverNode, oElementNode) {
                                    return scope.$root["workspaceTooltipPlacement"](oElementNode);
                                },
                                "html": true
                            });
                        });
                    }
                });
                // Attends la suppression du scope.
                scope.$on("$destroy", function () {
                    // Supprime le tooltip.
                    $(element)["popover"]("destroy");
                    // Supprime l'observateur.
                    clearObserver();
                });
            }
        }
    };
    vitisApp["compileProvider"].directive('appExtractionLayersColumn', vitisApp.appExtractionLayersColumnDrtv);
    
    /**
     * appExtractionRefUnusedAttributesColumn directive.
     * Mise en forme de la colonne "unused_attributes" dans la liste de l'onglet "Extraction".
     * @param {service} $translate Translate service.
     * @ngInject
     **/
    vitisApp.appExtractionRefUnusedAttributesColumnDrtv = function ($translate) {
        return {
            link: function (scope, element, attrs) {
                // 1er affichage ou tri de la liste : maj de la mise en forme.
                var clearObserver = attrs.$observe("appExtractionRefUnusedAttributesColumn", function (value) {
                    // Si le champ est vide : supprime l'icône.
                    if (scope["row"]["entity"][scope["col"]["field"]] == null || scope["row"]["entity"][scope["col"]["field"]] == "")
                        element[0].className = "";
                    else {
                        // Classes css (ui-grid + spécifiques).
                        element[0].className = "ui-grid-cell-contents info-icon";
                        // Traduction du titre et du contenu.
                        $translate(["EXTRACTION_REF_EXTRACTION_LAYER_UNUSED_ATTRIBUTES_TOOLTIP_TITLE"]).then(function (translations) {
                            // Création du "tooltip".
                            $(element)["popover"]({
                                "trigger": "hover",
                                "container": "body",
                                "title": function () {
                                    return translations["EXTRACTION_REF_EXTRACTION_LAYER_UNUSED_ATTRIBUTES_TOOLTIP_TITLE"];
                                },
                                "content": function () {
                                    var sValue = scope["row"]["entity"][scope["col"]["field"]];
                                    return sValue.replace(/;/g, "<br>");
                                },
                                // Placement du tooltip à gauche ou à droite suivant la position horizontale de l'élément.
                                "placement": function (oPopoverNode, oElementNode) {
                                    return scope.$root["workspaceTooltipPlacement"](oElementNode);
                                },
                                "html": true
                            });
                        });
                    }
                });
                // Attends la suppression du scope.
                scope.$on("$destroy", function () {
                    // Supprime le tooltip.
                    $(element)["popover"]("destroy");
                    // Supprime l'observateur.
                    clearObserver();
                });
            }
        }
    };
    vitisApp["compileProvider"].directive('appExtractionRefUnusedAttributesColumn', vitisApp.appExtractionRefUnusedAttributesColumnDrtv);
    
    angular.element(vitisApp.appMainDrtv).scope()["showModalSignUp"] = function () {
        $("#modal_sign_up").modal('open');
        
        var oFormValues = {};
        oFormValues["search"] = {};
        ajaxRequest({
            'method': 'GET',
            'url': propertiesSrvc["web_server_name"] + '/' + propertiesSrvc["application"] + "/modules/extraction/forms/sign_up.json",
            'success': function (response) {
                hideAjaxLoader();
                
                var oFormReaderScope = angular.element($("#modal_sign_up_form_container").children()).scope();
                oFormReaderScope.$evalAsync(function () {
                    oFormReaderScope['ctrl']['setDefinitionName']("search");
                    oFormReaderScope['ctrl']['setFormValues'](oFormValues);
                    oFormReaderScope['ctrl']['setFormDefinition'](response["data"]);
                    oFormReaderScope['ctrl']['loadForm']();
                });
                
                $this.$timeout_(function () {
                    //Materialize.updateTextFields();
                    $(".checkbox").removeClass("checkbox checkbox-margin");
                    
                    //$this.cssSelectFormDefinitionListener(oFormReaderScope);
                }, 100);
            },
            'error': function (response) {
                hideAjaxLoader();
                console.error("ERROR");
            }
        });
    };
});

vitisApp.on('appInitCtrlLoaded', function () {
    var $rootScope = angular.element(vitisApp.appMainDrtv).injector().get(["$rootScope"]);
    var envSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["envSrvc"]);
    var $translatePartialLoader = angular.element(vitisApp.appMainDrtv).injector().get(["$translatePartialLoader"]);
    var $translate = angular.element(vitisApp.appMainDrtv).injector().get(["$translate"]);
    // Chargement du template de login et des formulaires spécifiques à Extraction.
    //envSrvc["sLoginTemplate"] = "modules/calimap/templates/loginTpl.html";
    //envSrvc["sLoginForm"] = "modules/extraction/forms/extraction/login.json";
    //envSrvc["sForgottenPasswordForm"] = "modules/calimap/forms/login/forgotten_password.json";
    envSrvc["sSignUpForm"] = "modules/extraction/forms/login/sign_up.json";
    // Chargement du fichier less pour le login.
    //loadExternalCss(["modules/calimap/less/login.less"]);
    // Chargement des fichiers de langues du module.
    var clearListener = $rootScope.$on('endFormNgRepeat', function (event) {
        clearListener();
        $translatePartialLoader["addPart"]("modules/extraction/lang");
        $translate["refresh"]();
    });
});