<?php

/**
 * \file Extraction_depots.class.inc
 * \class Extraction_depots
 *
 * \author WAB <support.wab@veremes.com>.
 *
 *     \brief This file contains the Extraction_depots php class
 *
 * This class defines Rest Api to Vitis Extraction_depots
 *
 */
require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once 'Extraction_depot.class.inc';
require_once __DIR__ . '/../../class/wab_lib/BusinessObject.class.inc';
require_once __DIR__ . '/Extractions.class.inc';

class Extraction_depots extends ExtractionExtraction
{
    /**
     * @SWG\Definition(
     *   definition="/extraction_depots",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/extraction_depots")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Extraction_depots",
     *   description="Operations about Extraction_depots"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    public function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false)
    {
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    }

    /**
     * @SWG\Get(path="/extraction_depots",
     *   tags={"Extraction_depots"},
     *   summary="Get Extraction_depots",
     *   description="Request to get Extraction_depots",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="order_by",
     *     in="query",
     *     description="list of ordering fields",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="sort_order",
     *     in="query",
     *     description="sort_order",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="limit",
     *     in="query",
     *     description="number of element",
     *     required=false,
     *     type="integer",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="offset",
     *     in="query",
     *     description="index of first element",
     *     required=false,
     *     type="string",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="filter",
     *     in="query",
     *     description="filter results",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="distinct",
     *     in="query",
     *     description="delete duplicates",
     *     required=false,
     *     type="boolean"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="extraction_depot Response",
     *         @SWG\Schema(ref="#/definitions/extraction_depots")
     *     )
     *  )
     */

    /**
     * get Extraction_depots
     * @return  Extraction_depots
     */
    public function GET()
    {
        if (!empty($this->aPath[2])) {
            if ($this->aPath[2] == "Tree") {
                $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot';
                //Création du dossier extraction_depot
                if (!is_dir($sDirPath)) {
                    //Création du dossier du demandeur
                    if (!mkdir($sDirPath)) {
                        //Mettre une erreur dans le log d'erreur
                    }
                }
                $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"];
                if (!is_dir($sDirPath)) {
                    if (!mkdir($sDirPath)) {
                        //Mettre une erreur dans le log d'erreur
                    }
                }
                $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"] . "/Fichiers déposés";
                if (!is_dir($sDirPath)) {
                    if (!mkdir($sDirPath)) {
                        //Mettre une erreur dans le log d'erreur
                    }
                }
                $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"] . "/Résultats";
                if (!is_dir($sDirPath)) {
                    if (!mkdir($sDirPath)) {
                        //Mettre une erreur dans le log d'erreur
                    }
                }
                $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"];
                return json_encode($this->getFolderInfos($sDirPath));
            } elseif ($this->aPath[2] == "File") {
                $this->getWorkspaceFile();
            }
        }
    }

    /**
     * Get file.
     */
    public function getWorkspaceFile()
    {
        $aReturn   = array('status' => 1, 'message' => '');
        $sFilePath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"] . "/" . $this->aValues['file_name'];
        // Le fichier existe ?
        if (file_exists($sFilePath)) {
            header("Content-Type: application/octet-stream");
            header("Content-Transfer-Encoding: Binary");
            header("Content-disposition: attachment; filename=\"" . $this->aValues['file_name'] . "\"");
            header('Content-Length: ' . filesize($sFilePath));
            error_log(readfile($sFilePath));
        }
    }

    /**
     *
     * @param string $sDirectoryPath
     * @return array $aTree
     */
    public function getFolderInfos($sDirectoryPath)
    {

        if (is_dir($sDirectoryPath)) {
            $aContentDir = scandir($sDirectoryPath);
            $aTree       = $this->getFileInfos($sDirectoryPath);
            $aDir        = array();
            for ($i = 0; $i < count($aContentDir); $i++) {
                if ($aContentDir[$i] !== "." && $aContentDir[$i] !== "..") {
                    array_push($aDir, $this->getFolderInfos($sDirectoryPath . "/" . $aContentDir[$i]));
                }
            }

            $aTree["content"] = $aDir;
            $aTree["status"]  = 1;
            return $aTree;
        } else {
            return $this->getFileInfos($sDirectoryPath);
        }
    }

    /**
     *
     * @param string $sFilePath
     * @return array
     */
    public function getFileInfos($sFilePath)
    {
        $iFileSize = filesize($sFilePath);

        $sFileSize = $iFileSize . "octets";
        if ($iFileSize > 1024) {
            if ($iFileSize > 1024 * 1024) {
                if ($iFileSize > 1024 * 1024 * 1024) {
                    $sFileSize = "-";
                } else {
                    $sFileSize = (ceil($iFileSize / (1024 * 1024))) . "Mo";
                }
            } else {
                $sFileSize = (ceil($iFileSize / 1024)) . "Ko";
            }
        }

        $aFileName = explode(DIRECTORY_SEPARATOR, $sFilePath);

        return array("filename" => $aFileName[count($aFileName) - 1], "size" => $sFileSize, "lastModification" => date("d/m/Y H:i:s", filemtime($sFilePath)));
    }

    /**
     * @SWG\Delete(path="/[Class_name]s",
     *   tags={"[Class_name_upper]s"},
     *   summary="delete [Class_name_upper]s",
     *   description="Request to delete [Class_name_upper]s",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="idList",
     *     in="query",
     *     description="id of the [Class_name]",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="[Class_name] Response",
     *         @SWG\Schema(ref="#/definitions/[Class_name]s")
     *     )
     *  )
     */
    /**
     * @SWG\Delete(path="/[Class_name]s/{id_[Class_name]}",
     *   tags={"[Class_name_upper]s"},
     *   summary="delete [Class_name_upper]s",
     *   description="Request to delete [Class_name_upper]s",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="[Class_name_upper] token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_[Class_name]",
     *     in="path",
     *     description="id of the [Class_name_upper]s",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/[Class_name]s")
     *     )
     *  )
     */

    /**
     * delete [Class_name]s
     * @return id of [Class_name]s deleted or error object if a [Class_name]s is not deleted
     */
    public function DELETES()
    {
        $sDirPath = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"];
        //On supprime le fichier
        $sMessage = unlink($sDirPath . "/" . $this->aValues['file_name']);
        return $sMessage;
    }

    /**
     * @SWG\Put(path="/[Class_name]s/{id_[Class_name]}",
     *   tags={"[Class_name_upper]s"},
     *   summary="update [Class_name_upper]s",
     *   description="Request to update [Class_name_upper]s",
     *   operationId="PUT",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="[Class_name_upper] token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_[Class_name]",
     *     in="path",
     *     description="id of the [Class_name_upper]s",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/[Class_name]s")
     *     )
     *  )
     */

    /**
     * update [Class_name]s
     * @return id of [Class_name]s updated or error object if a [Class_name]s is not updated
     */
    public function PUT()
    {
        if (!empty($this->aValues["action"]) && $this->aValues["action"] == "Metadata") {
            $aReturn = $this->reintegrateMetadata();

            $aXmlRacineAttribute['status'] = $aReturn['status'];
            $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
        } else {
            $sDirName      = $this->aProperties['ws_data_dir'] . '/extraction/extraction_depot/' . $_SERVER["REMOTE_ADDR"];
            $error_message = "";

            // Si le dossier {id} n'existe pas (Important pour gérer les erreurs)
            if (!is_dir($sDirName)) {
                $error_message .= $sDirName . " does not exist";
                writeToErrorLog("WARNING: " . $error_message);

                $aXmlRacineAttribute['status'] = 0;
                $this->aFields                 = array();
                $this->aFields['errorMessage'] = $error_message;
                $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
                return $sMessage;
            }
            // Si le dossier {id}/Fichiers déposés n'existe pas (Important pour gérer les erreurs)
            if (!is_dir($sDirName . "/Fichiers déposés")) {
                $error_message .= $sDirName . "/Fichiers déposés does not exist";
                writeToErrorLog("WARNING: " . $error_message);

                $aXmlRacineAttribute['status'] = 0;
                $this->aFields                 = array();
                $this->aFields['errorMessage'] = $error_message;
                $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
                return $sMessage;
            }

            // Si le dossier {id}/form n'existe pas il est crée
            if (!is_dir($sDirName . "/Résultats")) {
                writeToErrorLog("WARNING: " . $sDirName . "/Résultats");
                @mkdir($sDirName . "/Résultats");
            }

            if (!empty($this->aValues["control_file_file"])) {
                $this->aValues["control_file"] = $this->aValues["control_file_name"];
            }
            if (!empty($this->aValues["control_file_file"])) {
                //if ($aReturn['sStatus'] == 1) {
                $aXmlRacineAttribute['status'] = 1;
                $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
                $sDirPath                      = $this->aProperties["upload_dir"] . "/" . getUniqRandomId();
                while (is_dir($sDirPath)) {
                    $sDirPath = $this->aProperties["upload_dir"] . "/" . getUniqRandomId();
                }
                if (!mkdir($sDirPath)) {
                    return false;
                }
                // Sauvegarde le nouveau fichier.
                $fp = fopen($sDirPath . '/' . utf8_decode($this->aValues["control_file_name"]), "w");
                fwrite($fp, $this->aValues["control_file_file"]);
                fclose($fp);

                $sExtension = '*.' . end(explode(".", $this->aValues["control_file_name"]));
                if (in_array($sExtension, explode("|", $this->aProperties['sig'])) or in_array($sExtension, explode("|", $this->aProperties['dao']))) {
                    $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
                    $this->oBO->insertOrder('EM0160AYYTK48', 'sourceDir=' . end(explode("/", $sDirPath)) . '/' . $this->aValues["input_deposit_name"] . '|sourceMotif=' . $sExtension . '|schema=' . $sSchemaContrainte . '|format=ACAD');
                } else {
                    //Si l'extension n'est pas bonne
                    $error_message .= $this->aValues["control_file_name"] . " format is not authaurized (". $sExtension . ")";
                    writeToErrorLog("WARNING: " . $error_message);

                    $aXmlRacineAttribute['status'] = 0;
                    $this->aFields                 = array();
                    $this->aFields['errorMessage'] = $error_message;
                    $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
                    return $sMessage;
                }
            }
        }

        return $sMessage;
    }

}
