<?php

/**
 * \file Formats.class.inc
 * \class Formats
 *
 * \author WAB <support.wab@veremes.com>.
 *
 * 	\brief This file contains the Formats php class
 *
 * This class defines Rest Api to Vitis Formats
 * 
 */
require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once 'Format.class.inc';


class Formats extends ExtractionExtraction {
    /**
     * @SWG\Definition(
     *   definition="/formats",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/formats")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Formats",
     *   description="Operations about Formats"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false){
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    }

/**
     * @SWG\Get(path="/formats",
     *   tags={"Formats"},
     *   summary="Get Formats",
     *   description="Request to get Formats",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="order_by",
     *     in="query",
     *     description="list of ordering fields",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="sort_order",
     *     in="query",
     *     description="sort_order",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="limit",
     *     in="query",
     *     description="number of element",
     *     required=false,
     *     type="integer",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="offset",
     *     in="query",
     *     description="index of first element",
     *     required=false,
     *     type="string",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="filter",
     *     in="query",
     *     description="filter results",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="distinct",
     *     in="query",
     *     description="delete duplicates",
     *     required=false,
     *     type="boolean"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="format Response",
     *         @SWG\Schema(ref="#/definitions/formats")
     *     )
     *  )
     */

    /**
     * get Formats
     * @return  Formats
     */
    function GET() {
        $aReturn = $this->genericGet($this->aProperties['schema_extraction'], "rt_format", "format_id");
        $sMessage = $aReturn['sMessage'];
        return $sMessage;
    }

/**
     * @SWG\Post(path="/formats",
     *   tags={"Formats"},
     *   summary="Add format",
     *   description="Request to add Formats",
     *   operationId="POST",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="format Response",
     *         @SWG\Schema(ref="#/definitions/formats")
     *     )
     *  )
     */

    /**
     * insert format
     * @return id of the format created
     */
    function POST() {
        $aReturn = $this->genericPost($this->aProperties['schema_extraction'], 'rt_format', '', 'format_id');
        
        return $aReturn['sMessage'];
    }

/**
     * @SWG\Put(path="/formats/{id_format}",
     *   tags={"Formats"},
     *   summary="update Formats",
     *   description="Request to update Formats",
     *   operationId="PUT",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Format token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_format",
     *     in="path",
     *     description="id of the Formats",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/formats")
     *     )
     *  )
     */

    /**
     * update formats
     * @return id of formats updated or error object if a formats is not updated
     */
    function PUT() {
        $aReturn = $this->genericPut($this->aProperties['schema_extraction'], 'rt_format', 'format_id');
        
        return $aReturn['sMessage'];
    }

/**
     * @SWG\Delete(path="/formats",
     *   tags={"Formats"},
     *   summary="delete Formats",
     *   description="Request to delete Formats",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="idList",
     *     in="query",
     *     description="id of the format",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="format Response",
     *         @SWG\Schema(ref="#/definitions/formats")
     *     )
     *  )
     */
    /**
     * @SWG\Delete(path="/formats/{id_format}",
     *   tags={"Formats"},
     *   summary="delete Formats",
     *   description="Request to delete Formats",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Format token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_format",
     *     in="path",
     *     description="id of the Formats",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/formats")
     *     )
     *  )
     */

    /**
     * delete formats
     * @return id of formats deleted or error object if a formats is not deleted
     */
    function DELETE() {
        $aReturn = $this->genericDelete($this->aProperties['schema_extraction'], 'rt_format', 'format_id');
        return $aReturn['sMessage'];
    }

	
}

?>